import * as React from 'react';
import React__default from 'react';

interface SelectionRange {
    start: number;
    end: number;
}

/**
 * The state of the text of the whole editor
 */
interface TextState {
    /**
     * All the text in the editor
     */
    text: string;
    /**
     * The section of the text that is selected
     */
    selection: SelectionRange;
}
interface TextController {
    /**
     * Replaces the current selection with the new text. This will make the new selectedText to be empty, the
     * selection start and selection end will be the same and will both point to the end
     * @param text Text that should replace the current selection
     */
    replaceSelection(text: string): TextState;
    /**
     * Selects the specified text range
     * @param selection
     */
    setSelectionRange(selection: SelectionRange): TextState;
    /**
     * Get the current text state
     */
    getState(): TextState;
}

interface ExecuteOptions {
    initialState: TextState;
    textApi: TextController;
}
interface Command {
    shouldUndo?: (options: Pick<ExecuteOptions, "initialState">) => boolean;
    execute: (options: ExecuteOptions) => void;
    undo?: (options: ExecuteOptions) => void;
}
interface CommandContext {
    type: string;
}
declare type CommandMap<CommandName extends string> = Record<CommandName, Command>;

declare const header: Command;

declare const bold: Command;

declare const italic: Command;

declare const strikethrough: Command;

declare const link: Command;

declare const quote: Command;

declare const codeBlock: Command;

declare const unorderedListCommand: Command;
declare const orderedListCommand: Command;
declare const checkedListCommand: Command;

declare const image: Command;

declare class CommandController<CommandName extends string> {
    private readonly textController;
    private readonly commandMap;
    /**
     * Indicates whether there is a command currently executing
     */
    isExecuting: boolean;
    constructor(textController: TextController, commandMap: CommandMap<CommandName>);
    executeCommand(commandName: CommandName, context?: CommandContext): Promise<void>;
}

declare class TextAreaTextController implements TextController {
    textAreaRef: React.RefObject<HTMLTextAreaElement>;
    constructor(textAreaRef: React.RefObject<HTMLTextAreaElement>);
    replaceSelection(text: string): TextState;
    setSelectionRange(selection: SelectionRange): TextState;
    getState(): TextState;
}

/**
 * A list of helpers for manipulating markdown text.
 * These helpers do not interface with a textarea. For that, see
 */
declare const textHelpers: {
    getSurroundingWord: (text: string, position: number) => SelectionRange;
    /**
     * If the cursor is inside a word and (selection.start === selection.end)
     * returns a new Selection where the whole word is selected
     * @param text
     * @param selection
     */
    selectWord: ({ text, selection }: TextState) => SelectionRange;
    /**
     *  Gets the number of line-breaks that would have to be inserted before the given 'startPosition'
     *  to make sure there's an empty line between 'startPosition' and the previous text
     */
    getBreaksNeededForEmptyLineBefore: (text: string | undefined, startPosition: number) => number;
    /**
     *  Gets the number of line-breaks that would have to be inserted after the given 'startPosition'
     *  to make sure there's an empty line between 'startPosition' and the next text
     */
    getBreaksNeededForEmptyLineAfter(text: string | undefined, startPosition: number): number;
    getSelectedText(textSection: TextState): string;
    getCharactersBeforeSelection(textState: TextState, characters: number): string;
    getCharactersAfterSelection(textState: TextState, characters: number): string;
};

declare const code: Command;

declare type UseTextAreaMarkdownEditorResult<CommandName extends string> = {
    ref: React__default.RefObject<HTMLTextAreaElement>;
    textController: TextController;
    commandController: CommandController<CommandName>;
};
declare type UseTextAreaMarkdownEditorOptions<CommandName extends string> = {
    commandMap: CommandMap<CommandName>;
};
declare function useTextAreaMarkdownEditor<CommandName extends string>(options: UseTextAreaMarkdownEditorOptions<CommandName>): UseTextAreaMarkdownEditorResult<CommandName>;

export { CommandController, TextAreaTextController, TextController, bold, checkedListCommand, code, codeBlock, header, image, italic, link, orderedListCommand, quote, strikethrough, textHelpers, unorderedListCommand, useTextAreaMarkdownEditor };
